# -*- coding: utf-8 -*-

#  /_/ _  _ _  /_      _
# / / /_'/ /_//\ /_/. /_//_/
#                    /   _/


"""
heroku3.py
~~~~~~~~~

:copyright: (c) 2013 by Heroku.
:license: MIT, see LICENSE for more details.
"""

# Meta.

__title__ = "heroku3"
__author__ = "Heroku"
__copyright__ = "Copyright 2013 Heroku"
__license__ = "MIT"

__version__ = "5.2.0"

# Module namespace.

# Project libraries
from .core import from_key  # noqa
